require(plyr)
require(dplyr)
require(tidyverse)
require(readxl) 
require(ggpubr)
#install.packages("BiocManager")
#BiocManager::install("pcaMethods")
require(pcaMethods)
require(ggplot2)
require(lemon)
require(xlsx)
require(purrr)
require(colorspace)
require(splancs)
#########################################################################################################################################
#############################################################################################################################################
################################### imputation-specific self-made function  ####################################################
#############################################################################################################################################
###############################################################################################################################################
# impute_classy separates into subsets based on group and then runs BPCA imputation of a subset of a provided dataset of cranial variables (cranial) 
## using the range of the data columns (range1) and the range for the specimen information (range2)
## it then combines the imputed subsets and returns a single imputed dataset
#### NOTE: the data columns will be log-transformed prior to imputation
impute_class<-function(cranial,range1,range2){
  # range1 represents the range for the data columns
  # range2 represents the label and other specimen information columns
  # cranial is the dataset
  ##################################################
  ### create individual data sets for each subfamily for imputation
  hadro<-subset(cranial, group=="Hadrosauridae"|group=="Iguanodontia"|group=="Hadrosauroidea")
  saur1<-subset(cranial, group=="Diplodocidae")
  saur2<-subset(cranial, group=="Camarasauridae"|group=="Brachiosauridae"|group=="Titanosaur")
  steg<-subset(cranial, group=="Stegosauridae")
  ank<-subset(cranial, group=="Ankylosauria")
  cer<-subset(cranial, group=="Ceratopsidae")
  ########################################################
  
  ##################################################
  ### takes data frame of certain dimensions(e.g. cranial_hadro[,8:19]) and imputes, retrieves data
  ##data frame must have data in columns 8 to 19 while columns 1 to 7 are documentation
  BPCA_data <- function(data,range1, range2){
    log_data<-log10(data[,range1] )# range represents the log-transformed data columns through numbers in square brackets with your first numerical column and your last numerical column, respectifully. 
    # use correlation matrix for PCA because SSI is a unitless measurement
    est_data<-pca(log_data,method="bpca",centre=FALSE, cor=TRUE)
    
    est_matrix<-est_data@completeObs # This will retreive the estimated matrix from the previous fucntion.
    fin<-cbind(data[,range2],est_matrix)
    #est_matrix # This will display your new estimated matrix.  You can  copy and paste it out of R if you want.
    
    return(fin)}
  
  ##### run imputation on individual data sets
  finCranHadro<-BPCA_data(hadro,range1, range2)
  finCranSaur1<-BPCA_data(saur1,range1, range2)
  finCranSaur2<-BPCA_data(saur2,range1, range2)
  finCranSteg<-BPCA_data(steg,range1, range2)
  finCranAnk<-BPCA_data(ank,range1, range2)
  finCranCer<-BPCA_data(cer,range1, range2)

  cranData<-rbind(finCranHadro,finCranSaur1,finCranSaur2,finCranSteg,finCranAnk,finCranCer)### combine the above imputed subsets into a single dataset 
  
  return(cranData) }

###############################################################################################################################################
#################################################### run the imputation #######################################################################
### import data 
setwd(dirname(rstudioapi::getActiveDocumentContext()$path)) # sets the working directory automatically to wherever your R script is saved.
mydata <- read_excel("Supplementary Tables 1-5_ecomorph occ Late J-K NA.xlsx", sheet = "STable2", skip=1) # North American Taxa
nonNA <-read_excel("Supplementary Tables 1-5_ecomorph occ Late J-K NA.xlsx", sheet = "STable3", skip=1) # non-North American Taxa

# impute missing values and obtain categorical information that will be needed later for analysis; note that all values obtained will be log10 transformed
cran<-impute_class(mydata, 8:19, 2:7) #NOTE: imputation occurs by group 


# impute the non-NA with the North American taxa but only keep the non-Na imputed results (don't want nonNA taxa influencing NA taxa)
specIDnonNA<- nonNA$`specimen ID`#obtain vector with all the non-NA taxa so can separate out the imputed nonNA
alldata<-data.frame(rbind(mydata, nonNA)) # combine nonNA and NA datasets for imputation
nonNAcran<-impute_class(alldata, 8:19, 1:7) #NOTE: imputation occurs by group 

nonNAcran<-nonNAcran[which( nonNAcran$specimen.ID %in% specIDnonNA)  , ]

# create a column indicating information on locality that can be used to separate out the taxa for plotting 
nonNAcran<-data.frame(cbind(loc= nonNAcran$comments, nonNAcran[,3:19]) )
cran<-data.frame( cbind( loc=rep("NoA", nrow(cran) ), cran[,2:18]) ) # have to use NoA because otherwise could have issues with "NA" (as in NA not filled cell indicator)

# combine cran and nonNAcran to construct a dataframe with all taxa for the overall analysis
allcran<- data.frame(rbind(cran, nonNAcran))

#################################################################################################################################
# obtain information that will be needed for plotting the PC scores and the loading plots
# PC_info(infotype, pcaAll, category)
# infotype is whether want the scores, pov or loadings; pcaAll is the list produced from conducting the principal components analysis; category is the columns of cran_res containing categorical information; 
PC_info<-function(infotype, pcaAll, category){
  names<-c("PC1", "PC2", "PC3", "PC4", "PC5", "PC6", "PC7", "PC8", "PC9","PC10","PC11", "PC12") #used to label the colummns for returned dataframe
  if(infotype=="scores"){
    pcaData<-data.frame(pcaAll$scores)# scores
    colnames(pcaData)<- names
    pcaData<-cbind(category,pcaData)# combine PCA axis values with the relevant information
  } # the scores from PCA
  
  if(infotype=="pov"){
    pcaData<-data.frame(matrix(nrow=1,ncol=12)) # new dataframe for the % of variance
    colnames(pcaData)<-names
    pcaData[1,] <- pcaAll$sdev^2/sum(pcaAll$sdev^2) #to get a vector of the % variance into the dataframe
    pcaData<-round(pcaData*100,2)  }# the percent of variation
  
  if(infotype=="loads"){
    pcaData<-data.frame(matrix(pcaAll$loadings, ncol=12))
    colnames(pcaData)<-names
    rownames(pcaData)<-c("as-mq","cs-mq", "mt-mq","dt-mq", "sw", "dh","ppb","oh","cp-jj", "sh","qb", "SSI") # corresponding variable names
  }# to get the loadings for the axes
  
  return(pcaData) }
###############################################################################################################################################

###############################################################################################################################################
#PCload_plots creates and combines the individual loadings plots into a single plot and returns this single plot when provided with the 
### loading information (='pcloads')
PCload_plots<-function(pcloads){
  theme_master<-function(base_size=12, base_family=""){
    theme_classic(base_size=base_size, base_family=base_family)%+replace%
      theme(
        axis.text=element_text(color="black"),
        axis.title.x=element_text(size=12, vjust=-0.25, color="black"),
        axis.text.x=element_text(size=10,angle=90,vjust=0.5, hjust=1, color="black",margin=margin(3,2,3,2)),
        axis.title.y=element_text(size=12, angle=90,  color="black"),
        axis.text.y=element_text(size=10,  color="black"),
        
        axis.ticks=element_line(color="black"),
        axis.ticks.length=unit(1, "mm"),
        
        axis.line=element_line(color="black", linetype="solid"),
        #panel.grid.minor.y=element_line(color="#DCDCDC", size=0.35),
        panel.grid.major.y=element_line(color="#C0C0C0", size=0.5),
        
        panel.background = element_blank(),
        panel.border=element_rect(fill=NA,color="black", size=0.5, linetype="solid"),
        plot.margin=unit(c(4,4,4,3),"mm"),
        
        legend.position = "none")}
  
  #meas<-c("as-mq","cs-mq", "mt-mq","dt-mq", "sw", "dh","ppb","oh","cp-jj", "sh","qb", "SSI")
  meas<-as.factor(rownames(pcloads)) #obtain name
  
  loadings<-data.frame(meas)
  loadings$load<-pcloads[,1]
  loadings$PC<-c(paste("PC", 1, sep=""))
  colnames(loadings)<-c("meas","load","axis")
  
  i<-2
  while(i<ncol(pcloads)){
    pcx<-data.frame(meas)
    pcx$load<-pcloads[,i]
    pcx$PC<-c(paste("PC", i, sep=""))
    colnames(pcx)<-c("meas","load","axis")
    loadings<-bind_rows(loadings,pcx) 
    #loadings<-rbind.data.frame(loadings,pcx) 
    i<-i+1 }
  
  loadings$cat<-NA
  loadings$cat[loadings$load>0]<-"positive"
  loadings$cat[loadings$load<0]<-"negative"
  
  loadings$meas<-factor(loadings$meas,levels=meas)# reorder the groupings
  loadings$axis<-factor(loadings$axis,levels=c("PC1","PC2","PC3","PC4","PC5","PC6","PC7","PC8","PC9","PC10","PC11","PC12"))# reorder the groupings
  
  line<-loadings
  line$load<-0
  
  c1<-rgb(red=0.8,green=0, blue=0)
  c2<-rgb(red=0,green=0.8, blue=0)
  
  b<- ggplot()+
    geom_col(loadings, mapping=aes(x=meas, y=load, fill=factor(cat)))+
    scale_color_manual(values=c("negative"=c1, "positive"=c2))+
    scale_fill_manual(values=c("negative"=c1, "positive"=c2))+
    scale_y_continuous(n.breaks=7,breaks = waiver(), minor_breaks = waiver())+
    theme_master()+
    facet_rep_wrap( ~loadings$axis, scales='free_y',repeat.tick.labels=T)+
    labs(x="measurement", y="weighting")   
  return(b)}
###############################################################################################################################################

#---------------------------------------------------------------------------------------------------------------#
# these are analyses concerning only the North American taxa
na.pca_res<-princomp(cran[,7:18], cor=T)  ## get PCA result list for whole dataset

#na.pcscores<-PC_info("scores", na.pca_res, cran[,1:6])# PC scores needed for plotting
#na.pcPOV<-PC_info("pov", na.pca_res, cran[,1:6]) #the amount of the total variation in the dataset accounted for by each PC axis


all.pca_res<-princomp(allcran[,7:18], cor=T)  ## get PCA result list for whole dataset

#all.pcscores<-PC_info("scores", all.pca_res, allcran[,1:6])# PC scores needed for plotting
#all.pcPOV<-PC_info("pov", all.pca_res, allcran[,1:6]) #the amount of the total variation in the dataset accounted for by each PC axis

#__________________________________________________________________________________________________________________________#
#pcaData = all.pcscores # the data frame
#x = all.pcscores$PC1
#y = all.pcscores$PC2
#theTime= "Early Cretaceous"  
#by.lcl="no" # if locality should be separated or not

#pcaData = na.pcscores
#x =na.pcscores$PC1
#y =na.pcscores$PC2
#theTime= "Late Jurassic"
#by.lcl= "no"
#function will plot any PC axis (y) against any other PC axis (x) where x and y are both column indices within the dataframe pcaData
pc_xy_plotter<-function(
  pcaData, # the pcscores with categorical information
  x, # the column of x variables
  y, # the column of y variables
  theTime,#specifies which time period/stage you want plotted with points
  by.lcl # if non-North American locality should be indicated or not
  ){
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
  # plot aesthetics based on clade and overall plot theme
  #                        "DiplodocidaeI" ,  "DiplodocidaeA" , "broadI" ,  "broadA",       "cerI" ,"cerA" ,        "igI"  , "igA" ,     "ank",      "steg" 
  coldf<- data.frame( oCol= c("#DC1FBD",        "#DC1FBD",       "#9932CC", "#9932CC",   "#C71585", "#C71585" ,  "#4682B4", "#4682B4",  "#2E8B57",  "#E10000"), 
                      #                 royal  purple      ,           purple      ,        #(dark magenta) ,         (blue) ,                     ,       red
                      
                      fCol= c(NA, "#BA55D3",                   NA, "#FF80F3",           NA, "#E21798" ,         NA, "#6495ED",       NA,         NA),
                      #          royal  purple      ,          purple           ,        #(dark magenta) ,        (blue) ,                     ,       red
                      
                      shapes= c(0,        22,                 5, 23,                  2, 25 ,                     1, 21,            8,        4) 
                      #         fillable square      ,     fillable diamond     ,    fillable triangle ,        # filled circle,    star  ,    x
  )
  rownames(coldf)<-c(   "DiplodocidaeI" ,  "DiplodocidaeA" , "broadI" ,  "broadA", "cerI" ,"cerA" , "igI"  , "igA" ,  "ank",  "steg"    )
  
  hullCols<-c( "broad"= "#9932CC", #(purple)
               "Diplodocidae" ="#DC1FBD",#(royal purple)
               "ig" ="#4682B4",#  (blue)
               "steg" = "#E10000" , # (red)
               "cer" = "#C71585" ,#(dark magenta)
               "ank" = "#2E8B57" # (dark blue-green) 
  ) # color info for convex hulls
  # modify colors for use in convex hulls
  mycols<-data.frame( clade= names(hullCols), 
                      origcol= hullCols,
                      callT = NA, # the outline/fill for the hulls drawn around clades across time
                      fallT = NA # the outline/fill for the hulls drawn around clades across time
  )
  mycols$callT <- lighten( mycols$origcol, 0.1, method="absolute" )# the outline/fill for the hulls drawn around clades across time
  mycols$fallT <- lighten( mycols$origcol, 0.1, method="absolute" )# the outline/fill for the hulls drawn around clades across time
  
  #plot theme
  theme_master<-function(base_size=12, base_family=""){
    theme_classic(base_size=base_size, base_family=base_family)%+replace%
      theme(axis.text=element_text(color="black"),
            axis.title.x=element_text(size=18, vjust=-0.5, color="black"),
            axis.text.x=element_text(size=16, color="black"),
            axis.title.y=element_text(size=18, angle=90, vjust=3, color="black"),
            axis.text.y=element_text(size=16, angle=90, vjust=2, color="black"),
            axis.ticks=element_line(color="black"),
            axis.ticks.length=unit(2, "mm"),
            panel.background = element_blank(),
            panel.border=element_rect(fill=NA, color="black", size=1, linetype="solid"),
            plot.margin = unit(c(3, 2, 4, 4), "mm"))} # for plot theme 
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
  forPlot<-data.frame(genus =as.factor(pcaData$genus), 
                      specimen = pcaData$specimen.ID,
                      PC1 = x, 
                      PC = y, 
                      group = as.factor(pcaData$group), 
                      time = as.factor(pcaData$time),
                      Ontogeny = as.factor(pcaData$Ontogeny), 
                      lcl = pcaData$loc, 
                      Hulls= rep(NA, nrow(pcaData) ), 
                      fills= rep(NA, nrow(pcaData) ))
  
  #### create column containing categorical information to use to construct convex hulls
  forPlot$Hulls[ which(forPlot$group == "Hadrosauridae" |forPlot$group =="Iguanodontia"| forPlot$group =="Hadrosauroidea" )  ]<-"ig"
  forPlot$Hulls[ which(forPlot$group == "Diplodocidae")  ]<-"Diplodocidae"
  forPlot$Hulls[ which(forPlot$group == "Camarasauridae"|forPlot$group=="Brachiosauridae"|forPlot$group=="Titanosaur" )  ]<-"broad"
  forPlot$Hulls[ which(forPlot$group == "Ceratopsidae" )  ]<-"cer"
  forPlot$Hulls[ which(forPlot$group == "Ankylosauria")  ]<-"ank" 
  forPlot$Hulls[ which(forPlot$group == "Stegosauridae" )  ]<-"steg" 
  
  # to create fill conditions based on ontogeny to be used to plot the points
  forPlot$fills[ forPlot$Ontogeny !="adult" & forPlot$Hulls=="ig" ]<- "igI"
  forPlot$fills[ forPlot$Ontogeny =="adult" & forPlot$Hulls=="ig" ]<-"igA"
  
  forPlot$fills[ forPlot$Ontogeny !="adult" & forPlot$Hulls=="Diplodocidae" ]<-"DiplodocidaeI"
  forPlot$fills[ forPlot$Ontogeny =="adult" & forPlot$Hulls=="Diplodocidae" ]<-"DiplodocidaeA"
  
  forPlot$fills[ forPlot$Ontogeny !="adult" & forPlot$Hulls=="broad" ]<-"broadI"
  forPlot$fills[ forPlot$Ontogeny =="adult" & forPlot$Hulls=="broad" ]<-"broadA"
  
  forPlot$fills[ forPlot$Hulls=="steg" ]<-"steg"
  
  forPlot$fills[ forPlot$Ontogeny !="adult" & forPlot$Hulls=="cer" ]<-"cerI"
  forPlot$fills[ forPlot$Ontogeny =="adult" & forPlot$Hulls=="cer" ]<-"cerA"
  
  forPlot$fills[ forPlot$Hulls=="ank" ]<-"ank"
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
  
  ###########################################################################################################
  # calculate convex hulls 
  cvx_hull_finder<-function(forPlot){
    find_hull<-function(df) df[chull(df$PC1,df$PC),] 
    
    #===================================================================================================================#
    ### obtain convex hulls and find the area for each ###
    lvl<-levels(factor(forPlot$Hulls))# hull levels to create
    myhulls<-list()
    areadf<-data.frame(clade= rep(0, length(lvl)) , area= rep(0, length(lvl)) )
    
    i<-1
    while( i < length(lvl)+1 ){
      aclade<- lvl[i] #level of Hulls calculating the convex hull for
      h1<-subset(forPlot, forPlot$Hulls==aclade) # get subset in order to start calculating the convex hulls
      h1<-find_hull(h1) # find convex hull for the group
      myhulls[[aclade]]<-h1
      
      areadf[i,1]<-aclade
      a1<- as.numeric(splancs:: areapl(cbind(h1[,3],h1[,4]) ))  # find area of polygon and input that into a data frame
      areadf[i,2]<-a1
      i<-i+1      }
    #===================================================================================================================#
    areadf<-areadf%>%arrange (-area) # order from largest to smallest area
    myhulls<- myhulls[areadf$clade]
    
    return(myhulls)  } # finds convex hulls and returns a list of them to be used in order of greatest to smallest area
  ###########################################################################################################
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
  somehulls<-cvx_hull_finder(forPlot) # calculate convex hulls for clades across time
  p1<-ggplot() +geom_blank() # create blank plot
  
  i<-1
  clades<-data.frame(names(somehulls))
  while(i < nrow(clades)+1 ){
    ahull<-somehulls[[  clades[i,1] ]]
    myI<-which(mycols$clade %in% paste(clades[i,1]) ) # get index for the color from the mycols dataframe
    acol<-c(paste(mycols$callT[myI]))
    afill<-c(paste(mycols$fallT[myI]))
    p1<-p1+ geom_polygon(data=ahull, aes(x=PC1,y=PC), fill= acol,  color= acol, alpha=0.3,  lwd=0.8, show.legend=F)
    i<-i+1  }
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
  myorder<-c("DiplodocidaeI", "DiplodocidaeA", "broadI", "broadA", "cerI", "cerA", "igI", "igA", "ank", "steg" )
  forPlot$fills<-factor(forPlot$fills,levels=myorder)# reorder the groupings
  # create plots only containing taxa from given time period
  forPoints<-subset(forPlot, forPlot$time==theTime) # subset for the annotation layer of points
  
  #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
  ######## count number of specimens in each group
  num<-forPoints%>%
    count(fills)%>% data.frame()%>% ungroup()# provides number of observations per group in the form of a data frame for use in the legend

  num$val<-num$fills
  num<-num%>%
    mutate(val =
      case_when(
      fills == "DiplodocidaeI"~ paste("immature diplodocid (n=", num[which( num$fills =="DiplodocidaeI" ),2], ")", sep="") , 
      fills == "DiplodocidaeA"~ paste("mature diplodocid (n=", num[which( num$fills =="DiplodocidaeA" ),2], ")", sep="") ,  
      fills =="broadI"~ paste("immature non-diplodocid sauropod (n=", num[which( num$fills =="broadI" ),2], ")", sep="") ,  
      fills == "broadA"~ paste("mature non-diplodocid sauropod (n=", num[which( num$fills =="broadA" ),2], ")", sep="") , 
      fills == "cerI"~ paste("immature ceratopsid (n=", num[which( num$fills =="cerI" ),2], ")", sep="") , 
      fills == "cerA" ~ paste("mature ceratopsid (n=", num[which( num$fills =="cerA" ),2], ")", sep="") , 
      fills ==  "igI" ~ paste("immature iguanodontian (n=", num[which( num$fills =="igI" ),2], ")", sep="") , 
      fills =="igA"~ paste("mature iguanodontian (n=", num[which( num$fills =="igA" ),2], ")", sep="") ,  
      fills == "ank"~ paste("ankylosaur (n=", num[which( num$fills =="ank" ),2], ")", sep="") , 
      fills ==  "steg" ~ paste("stegosaur (n=", num[which( num$fills =="steg" ),2], ")", sep="")         )     )

  numVals<-num$val # create vector containing sample size info
  numVals<-setNames(numVals,num$fills) # create names for vector
    #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
  
  coldf1<-coldf[which(rownames(coldf) %in% num$fills),] # this removes aesthetic info that is not needed to prevent it from being included in the legend
  
  p<-p1 + 
    geom_point(data=forPoints,mapping=aes(x=PC1,y=PC,shape=fills, color=fills, fill=fills),size=5, stroke=1.2 )+ # add points onto the plot
    scale_x_continuous(expand= expansion(mult = c(0.15, 0.15)))+
    scale_y_continuous(expand= expansion(mult = c(0.1, 0.2)))+
    scale_color_manual( values = coldf1[['oCol']] %>% setNames ( rownames(coldf1)), labels = numVals, drop=T )+
    scale_fill_manual( values = coldf1[['fCol']] %>% setNames ( rownames(coldf1)), labels = numVals, drop=T )+
    scale_shape_manual( values = coldf1[['shapes']] %>% setNames ( rownames(coldf1)), labels = numVals , drop=T)+
    
    labs(color="legend", shape="legend",fill="legend")+
    theme_master()+
    theme(legend.text=element_text(color="black",size=18),legend.background=element_rect(color="black",size=0.8),legend.position="top", legend.title = element_blank() )+
    guides(shape = guide_legend(override.aes = list(size = 4, stroke=1.5),ncol=3),
           fill=guide_legend(ncol=3),
           color=guide_legend(ncol=3) )

  if( by.lcl== "yes"){
    forPoints$lcl1<-ifelse( forPoints$lcl == "NoA",  "", sapply(strsplit(forPoints$lcl,""), `[`, 1) )# produces text labels to be placed over top of points if needed
    p<-p+
      geom_text(data=forPoints, aes(x=PC1,y=PC, label=lcl1), check_overlap = F)  } # if also have specimens from various localities
  
  return(p)  }
#__________________________________________________________________________________________________________________________#

#________________________________________________________________________________#
###############################################################################################################################################  
# takes a data frame, PCA results and obtains PC plots 1 through 4 for the given time and returns them
# this function calls the pc_xy_plotter and PC_info functions
ecomorphospace<-function(pca_res, # list of results from pca
                         cranaes, # the aesthetic columns for the dataset to be used in finding the scores and pov with the PC_info function
                         by.lcl # if should include locality indicators for non north american taxa
                         ){
  #-----------------------------------------------------------------------------------------------#
  # find the pcscores and pcPOV to be used to construct the plots
  pcscores<-PC_info("scores", pca_res, cranaes)# PC scores needed for plotting
  pcPOV<-PC_info("pov", pca_res, cranaes) #the amount of the total variation in the dataset accounted for by each PC axis
  #-----------------------------------------------------------------------------------------------#
  time <-c("Late Jurassic", "Early Cretaceous", "Late Cretaceous")
    # produce plots
  plot_list1<-list()
  plot_list2<-plot_list1
  i<-1
  while( i< length(time)+1){ #length(time)
    
    theTime = time[i]
    
    p1<-pc_xy_plotter(pcscores, pcscores$PC1, pcscores$PC2, theTime, by.lcl )+  xlab(paste("PC1 (", round(pcPOV$PC1,1),"%)", sep=""))+  ylab(paste("PC2 (", round(pcPOV$PC2,1),"%)", sep="")) ## PC1 PC2
    p2<-pc_xy_plotter(pcscores, pcscores$PC1, pcscores$PC3, theTime, by.lcl )+  xlab(paste("PC1 (", round(pcPOV$PC1,1),"%)", sep=""))+  ylab(paste("PC3 (", round(pcPOV$PC3,1),"%)", sep="")) ## PC1 PC3
    p3<-pc_xy_plotter(pcscores, pcscores$PC1, pcscores$PC4, theTime, by.lcl )+  xlab(paste("PC1 (", round(pcPOV$PC1,1),"%)", sep=""))+  ylab(paste("PC4 (", round(pcPOV$PC4,1),"%)", sep="")) ## PC1 PC4
    pa<-ggarrange(p1,p2,  ncol=2, nrow=1, common.legend = T) # first 3 PCs in one object
    
    pa<-pa/patchwork::plot_spacer()+ patchwork::plot_layout(widths=c(1,1),  heights=c(1,0.05 ) )
    p3<-p3/patchwork::plot_spacer()+ patchwork::plot_layout(widths=c(1,1),  heights=c(1,0.05 ) )

    
    plot_list1[[i]]<-pa # PC1 vs PC2 and PC1 vs PC3
    plot_list2[[i]]<-p3 # PC1 vs PC4
    i<-i+1   }
  
  p1<- ggarrange(plot_list1[[3]],plot_list1[[2]],plot_list1[[1]] , 
                 nrow=3, labels= paste(LETTERS[1:length(time)], ")", sep=""),
                 font.label = list(size = 20, color = "black", face = "bold") )

  p2<- ggarrange(plot_list2[[3]],plot_list2[[2]],plot_list2[[1]] , 
                 nrow=3, labels= paste(LETTERS[1:length(time)], ")", sep=""),
                 font.label = list(size = 20, color = "black", face = "bold") )
  
  my_plots<-list(p1,p2)
  names(my_plots)<-c("PC1to3", "PC1v4")
  
  return(my_plots)}
#________________________________________________________________________________#


# obtain load plots
na.load_plot<-PCload_plots( PC_info("loads", na.pca_res ,cran[,1:6]) ) +
  theme(plot.margin=unit(c(4,4,4,3),"mm"), plot.title= element_text(size=16,hjust=0.01, face="bold"))
na.load_plot # supplementary figure 3


na.my.plots<-ecomorphospace(pca_res=na.pca_res, 
                            cranaes=cran[,1:6], 
                            by.lcl= "no")

na.my.plots[[1]]# figure 2
na.my.plots[[2]]# Supplementary figure 2

# export plots
ggsave(plot=na.my.plots[[1]],"Figure_2.svg",device="svg", width=620, height=700, units="mm",dpi =1000, scale=0.6)
dev.off()# for plots with PC1vPC2 and PC1vPC3

ggsave(plot=na.my.plots[[2]],"Supplementary_Figure_2.svg",device="svg", width=500, height=750, units="mm",dpi =1000, scale=0.6)
dev.off()# for plots with PC1vPC4

ggsave(plot=na.load_plot,"Supplementary_Figure_3.svg",device="svg", width=550, height=450, units="mm",dpi =1000, scale=0.6)
dev.off()
#________________________________________________________________________________#

# obtain load plots
all.load_plot<-PCload_plots( PC_info("loads", all.pca_res ,allcran[,1:6]) ) +
  theme(plot.margin=unit(c(4,4,4,3),"mm"), plot.title= element_text(size=16,hjust=0.01, face="bold"))
all.load_plot # supplementary figure 5


all.my.plots<-ecomorphospace(pca_res=all.pca_res, 
                            cranaes=allcran[,1:6], 
                            by.lcl= "yes")

all.my.plots[[1]]# figure 3
all.my.plots[[2]]# Supplementary figure 4

# export plots
ggsave(plot=all.my.plots[[1]],"Figure_3.svg",device="svg", width=620, height=700, units="mm",dpi =1000, scale=0.6)
dev.off()# for plots with PC1vPC2 and PC1vPC3

ggsave(plot=all.my.plots[[2]],"Supplementary_Figure_4.svg",device="svg", width=555, height=750, units="mm",dpi =1000, scale=0.6)
dev.off()# for plots with PC1vPC4

ggsave(plot=all.load_plot,"Supplementary_Figure_5.svg",device="svg", width=550, height=450, units="mm",dpi =1000, scale=0.6)
dev.off()
################################################################################################################################



########################################################################################################################################################
########################################################################################################################################################
######################################################             NPMANOVA                      #######################################################
########################################################################################################################################################
########################################################################################################################################################
require(vegan)
require(harmonicmeanp)

###############################################################################################################################################  
##this function randomly selects the same number of specimens from each group (selected based on the number of specimens from the smallest group)
### and puts these specimens into a single data frame for further analysis 
### this function is designed to randomly subsample from a data frame containing either two or three groupings
## my_range specifies the column indices containing the pcscores
rared<-function(df, my_range){
  df1<-data.frame(df$group)
  df1<-distinct(df1) #identify if have two or three subgroupings
  
  if(nrow(df1)==2){
    g1<-subset(df,group==df1[1,]) 
    g2<-subset(df,group==df1[2,])
    m1<-nrow(g1)
    m2<-nrow(g2)
    maximum<-min(m1,m2)# find maximum number of specimens that can be compared
    # select the above maximum number of specimens from each group randomly
    group1<-sample_n(g1, size=maximum, replace=FALSE)
    group2<-sample_n(g2, size=maximum, replace=FALSE)
    data1<-rbind(group1, group2)# form a new data frame from the selected specimens
  } # if there are two groupings
  
  if(nrow(df1)!=2){
    ### g's are matrices or data.frames
    g1<-subset(df,group==df1[1,]) 
    g2<-subset(df,group==df1[2,])
    g3<-subset(df,group==df1[3,])
    m1<-nrow(g1)
    m2<-nrow(g2)
    m3<-nrow(g3)
    maximum<-min(m1,m2,m3)# find maximum number of specimens that can be compared
    # select the above maximum number of specimens from each group randomly
    group1<-sample_n(g1, size=maximum, replace=FALSE)
    group2<-sample_n(g2, size=maximum, replace=FALSE)
    group3<-sample_n(g3, size=maximum, replace=FALSE)
    data1<-rbind(group1, group2,group3)# form a new data frame from the selected specimens
  } # if there are three groupings
  
  r<-cbind(data1$group,data1[,my_range]) # "group" column and the columns containing PC1 to PC4 which individually account for > 5% of the total variation
  colnames(r)<-c("family", names(data1[,my_range]))
  
  return(r) } # function that conducts rarifaction
#based on the weighting, the first 10 PC axes are going to be used in the NPMANOVA because each >1%

#####################################################################################################################

#####################################################################################################################
#### this function conducts at least one npmanova
## the_data - data frame
## ranges - column indices in the 'the_data' data frame containing the pc scores
## nit - number of iterations
## pair - if should conduct pairwise comparisons (if =Y then conducts pairwise comparisons for later use)

### conducts an omnibus test on a given set of PC scores run over nit intervals using a data frame containing a clade column and 
#### PC scores which are defined by the column indices 'range' (the data frame provided is called 'test')
#### a list of dataframes/values is returned: 
##### for the omnibus test (results of each iteration): model_omnibus, residual_omnibus, and total_omnibus, combined_om_p (the harmonic mean p value for the omnibus tests)
##### for the pairwise (if run): pair_combined (a column for the uncorrected harmonic mean p values and a column for Holm corrected harmonic mean p values) 
my_npmanova<-function(the_data, pair, nit, ranges){
  
  ## create empty dataframe for the results from the overall omnibus test to be saved for reference later if needed
  omnibus<-data.frame(matrix(nrow=4,ncol=7))
  colnames(omnibus)<-c("test","rowID","df", "SS", "R2", "F", "pr(>F)")
  all_omnibus<-omnibus
  p.omnibus<-data.frame(matrix(nrow=1,ncol=nit))
  
  
  if(pair=="Y"){
    ## create data frames for the results of the pairwise comparisons
    comparisons<-data.frame(matrix(data=c( c("iguanodontians","iguanodontians", "diplodocids"), c("diplodocids","non-diplodocids","non-diplodocids") ), nrow=3, ncol=2))
    
    pair1<-data.frame(matrix(nrow=4,ncol=7))
    colnames(pair1)<-c("test","rowID","df", "SS", "R2", "F", "pr(>F)")
    all_pair1<-pair1
    
    all_pair2<-pair1
    all_pair3<-pair1
    
    p.pair<-data.frame(matrix(nrow=nrow(comparisons),ncol=nit))
    rownames(p.pair)<-c(paste(comparisons[,1], "vs", comparisons[,2], sep=" ") )
    colnames(p.pair)<-seq(from=1, to =nit, by=1)     }
  
  i<-1
  while(i<nit+1){
    rare<-data.frame(rared(the_data, ranges))# conduct rarifaction to get the dataset to conduct investigations
    new_range<-2:ncol(rare) # obtain range of columns containing pcscores
    
    ############  omnibus test  #################
    ## want to use adonis2 for significance testing because it can perform overall tests. adonis performs a sequential test of terms
    x1<-adonis2(rare[,new_range]~family,  method="mahalanobis", by=NULL, data=rare)# conduct overall omnibus test on a randomly selected subset of the data using 
    
    test.name<-paste("test ",i, sep="")
    
    omnibus[1,]<-c(test.name, "model", round(x1[1,],3))
    omnibus[2,]<-c(test.name, "residual", round(x1[2,],3))
    omnibus[3,]<-c(test.name, "total", round(x1[3,],3))
    omnibus[4,]<-rep("-",7)
    
    p.omnibus[1,i]<-x1$`Pr(>F)`[1] #pr(>F) ; input p-value into a data frame
    
    #----------------------- combine anova tables into one data frame format -----------------#
    if(i==1){    all_omnibus<-omnibus   }
    if(i>1){ all_omnibus<-rbind(all_omnibus, omnibus)    } 
    #-----------------------------------------------------------------------------------------#
    
    
    #===========================================================================================================#
    #================================= Pairwise comparisons  ===================================================#
    
    if(pair=="Y"){
      ## this actually runs the pairwise comparisons one by one
      rw<-1
      while(rw< nrow(comparisons)+1){
        subs <-data.frame (subset(rare, family==comparisons$X1[rw] | family==comparisons$X2[rw] ) )# create subset containing two groups for pairwise comparison
        new_range<-2:ncol(subs) # obtain range of columns containing pcscores
        x2<-adonis2(subs[,new_range]~family, permutations=1000, method="mahalanobis", by=NULL, data=subs)# conduct overall omnibus test on a randomly selected subset of the data using 
        
        test.name<-paste(comparisons$X1[rw], "vs", comparisons$X2[rw],i, sep=" ")
        pair1[1,]<-c(test.name, "model", round(x2[1,],3))
        pair1[2,]<-c(test.name, "residual", round(x2[2,],3))
        pair1[3,]<-c(test.name, "total", round(x2[3,],3))
        pair1[4,]<-rep("-",7)
        
        p.pair[rw,i]<-x2$`Pr(>F)`[1] #pr(>F) ; input p-value into a data frame
        
        #----------------------- combine anova tables into one data frame format -----------------#
        if (rw==1){
          if(i==1){    all_pair1<-pair1   }
          if(i>1){ all_pair1<-rbind(all_pair1, pair1)    }         } # for first of 3 pairwise comparisons
        
        if (rw==2){
          if(i==1){    all_pair2<-pair1   }
          if(i>1){ all_pair2<-rbind(all_pair2, pair1)    }         } # for second of 3 pairwise comparisons
        
        if (rw==3){
          if(i==1){    all_pair3<-pair1   }
          if(i>1){ all_pair3<-rbind(all_pair3, pair1)    }         } # for third of 3 pairwise comparisons
        
        #-----------------------------------------------------------------------------------------#
        rw<-rw+1  } 
    }
    
    #===========================================================================================================#
    
    i<-i+1 } # run the rarefaction and then NPMANOVA using the mahalanobis distance and Holm correction
  
  omnibus_pval<-p.omnibus[1,]# vector containing the p-values from all iterations to be combined into a single p-value
  combined_om_p<-round(p.hmp(omnibus_pval, L = nit),3) # obtain harmonic mean p value
  
  # this cleans up and averages the pairwise comparisons only if the omnibus results are significant
  if(pair=="Y"){
    pair_combined<-data.frame(p.pair[,1:2]) # a data frame to put the combined p-values from the pairwise comparisons into prior to adjusting the p-valuesdata.frame(p.val[,1:2]) # a data frame to put the combined p-values from the pairwise comparisons into prior to adjusting the p-values
    colnames(pair_combined)<-c("uncorrected", "Holm-corrected")
    
    pair_combined[1,1]<-p.hmp( p.pair[1,] , L = nit)# p-value for the pairwise comparisons between iguanodontians vs diplodocids
    pair_combined[2,1]<-p.hmp( p.pair[2,] , L = nit)# p-value for the pairwise comparisons between iguanodontians vs non-diplodocids
    pair_combined[3,1]<-p.hmp( p.pair[3,] , L = nit)# p-value for the pairwise comparisons between sauropod groups
    
    pair_combined[,2]<- p.adjust(pair_combined[,1], method="holm")
    pair_combined<-round(pair_combined,3) # round to 3 significant digits
  }
  
  ##############################################################################################
  ####################### create list of information to be returned ############################
  if(pair=="N"){
    my_res<-list(all_omnibus, combined_om_p) # list of results to be returned
    names(my_res)<-c("omnibus","combined_om_p") # name each result so it can be selected later if desired
  }# does not contain information from pairwise comparisons
  
  if(pair=="Y"){
    my_res<-list(all_omnibus, combined_om_p, all_pair1, all_pair2, all_pair3, pair_combined) # list of results to be returned
    names(my_res)<-c("omnibus","combined_om_p","all_pair1","all_pair2","all_pair3","pair_combined") # name each result so it can be selected later if desired
  } # contains information from pairwise comparisons
  
  return(my_res)    }

#####################################################################################################################
#####################################################################################################################

#cranaes= cran[,1:6]
#pca_res= na.pca_res
# need PC_info function, my_npmanova function, and rared function
#
 run.my.npmanova<-function(
   pca_res, # list of results from the PCA
   cranaes # the columns containing the aesthetic information that will be passed on to PC_info
   ) {
   # find pcscores of pca test and pcPOV using the PC_info function
   pcscores = PC_info("scores", pca_res, cranaes)
   pcPOV =PC_info("pov", pca_res, cranaes)
   
   #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++# 
   # based on the PCA want to compare stegosaurs/ankylosaurs and sauropods/iguanodontians
   # simplify factor levels to reflect basic groupings
   pcscores$grouping<-pcscores$group
   pcscores$group[pcscores$grouping=="Iguanodontia"|pcscores$grouping=="Hadrosauroidea"|pcscores$grouping=="Hadrosauridae"]<-"iguanodontians"
   pcscores$group[pcscores$grouping=="Diplodocidae" ]<-"diplodocids"
   pcscores$group[pcscores$grouping=="Camarasauridae"|pcscores$grouping=="Brachiosauridae"|pcscores$grouping=="Titanosaur" ]<-"non-diplodocids"
   pcscores$group[pcscores$grouping=="Stegosauridae" ]<-"stegosaurids"
   pcscores$group[pcscores$grouping=="Ankylosauria" ]<-"ankylosaurs"
   
   ## create subset data frames containing only the groups that are to be compared to each other
   test1<-subset(pcscores,pcscores$group=="stegosaurids"|pcscores$group=="ankylosaurs" ) # for comparison of stegosaurids vs ankylosaurs
   test2<-subset(test1,test1$time!="Late Cretaceous") # Late Jurassic and Early Cretaceous forms only
   
   test3<-subset(pcscores,pcscores$group=="iguanodontians"|pcscores$group=="diplodocids"|pcscores$group=="non-diplodocids" ) # for comparison of iguanodontians and sauropods
   test4a<-subset(test3,test3$time!="Late Cretaceous") # select only Late Jurassic and Early Cretaceous iguanodontians and sauropods
   test4b<-subset(test3,test3$time=="Late Cretaceous"& test3$Ontogeny!="adult") # Late Cretaceous non-adult hadrosaurids (based on PCA seem like a reasonable proxy for more basal iguanodontians)
   test4<-data.frame(rbind(test4a,test4b))
   #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
   
   ## call the my_npmanova function to conduct statistical analyses on the various subsets produced above
   ### note range of PC scores only selects PCs1-4 as these individually account for more than 5% of the total variation
   pcs.look<-names(pcPOV)[which(pcPOV[1,]> 5)] # get the column names for PCs which account for >5 % of the variation to conduct the stats on
   pcs.incl<-which(names(pcscores) %in% pcs.look) # this obtains the column indices for the PC axes of interest
   
   #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
   #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
   ####################### comparisons involving stegosaurids and ankylosaurs ####################### 
   res1<-my_npmanova(test1, "N", 1000, pcs.incl) # conduct npmanova comparing stegosaurids to ankylosaurs from all time periods
   #test1_p<-res1$combined_om_p # obtain the omnibus harmonic mean p-value for test 1
   
   res2<-my_npmanova(test2, "N", 1000, pcs.incl) # conduct npmanova comparing stegosaurids to ankylosaurs from Late Jurassic and Early Cretaceous
   #test2_p<-res2$combined_om_p # obtain the omnibus harmonic mean p-value for test 2
   #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
   
   ####################### comparisons involving iguanodontians and sauropods ####################### 
   res3<-my_npmanova(test3, "Y", 1000, pcs.incl) # conduct npmanova comparing iguanodontians and sauropods from all time periods
   #test3_p<-res3$combined_om_p # obtain the omnibus harmonic mean p-value for test 3
   pair_comb3<-res3$pair_combined # corrected and uncorrected harmonic mean p-values for each pairwise comparison for test 3
   
   res4<-my_npmanova(test4, "Y", 1000, pcs.incl) # conduct npmanova comparing iguanodontians and sauropods from Late Jurassic and Early Cretaceous and immature Late Cretaceous hadrosaurids
   #test4_p<-res4$combined_om_p # obtain the omnibus harmonic mean p-value for test 4
   pair_comb4<-res4$pair_combined # corrected and uncorrected harmonic mean p-values for each pairwise comparison for test 4
   #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
   #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
   
   # create data frame containing omnibus test harmonic mean p-values
   omnibus_results<-data.frame(matrix(ncol=2, nrow=4))
   colnames(omnibus_results)<-c("test.name","hmp.value")
   omnibus_results$test.name<-c("stegosaurs vs ankylosaurs all time","stegosaurs vs ankylosaurs Late Jurassic and Early Cretaceous", 
                                "iguanodontians vs sauropods all time", "iguanodontians vs sauropods Late Jurassic and Early Cretaceous")
   omnibus_results$hmp.value<-c(res1$combined_om_p,res2$combined_om_p, res3$combined_om_p, res4$combined_om_p)
   
   emptycol<-rep(NA, nrow(res3$all_pair1)) # empty column to space between pairwise comparison information
   res3pairwise<-cbind(res3$all_pair1, " "=emptycol,res3$all_pair2, " "=emptycol, res3$all_pair3)
   
   emptycol<-rep(NA, nrow(res4$all_pair1)) # empty column to space between pairwise comparison information
   res4pairwise<-cbind(res4$all_pair1, " "=emptycol,res4$all_pair2, " "=emptycol, res4$all_pair3)
   
   #=============================================================================================# 
   # prepare everything and return
   my.results<-list(
     res1$omnibus, #results of NPMANOVA of stegosaurs vs ankylosaurs across time
     res2$omnibus, #results of NPMANOVA of stegosaurs vs ankylosaurs Late Jurassic and Early Cretaceous
     res3$omnibus, #results of NPMANOVA of iguanodontians and sauropods from all time
     res4$omnibus, #results of NPMANOVA of iguanodontians and sauropods from the Late Jurassic and Early Cretaceous
     res3pairwise, #results of NPMANOVA of pairwise comparisons between iguanodontians and sauropods from the Late Jurassic and Early Cretaceous
     res4pairwise,  #results of NPMANOVA of pairwise comparisons between iguanodontians and sauropods from the Late Jurassic and Early Cretaceous
     omnibus_results, # reported in the results section text
     pair_comb3, # reported in table 2 (NA) or 4 (non-NA)
     pair_comb4 # reported in table 3 (NA) or 5 (non-NA)
   )
   
   names(my.results)<-c(
     'res1Omn', 
     'res2Omn', 
     "res3Omn", 
     "res4Omn", 
     'res3Pair', 
     "res4Pair",
     "omnRes", 
     "pairComb3",
     "pairComb4" 
   )
   return(my.results)
   #=============================================================================================# 
   
   
   
 }

##############################################################################################
##################### save combined results of statistical tests #############################
na.np<-run.my.npmanova(pca_res = na.pca_res, cranaes = cran[,1:6])
all.np<-run.my.npmanova(pca_res = all.pca_res, cranaes = allcran[,1:6])

require(xlsx)

### export degrees of freedom, test-statistics and p-values for each individual test as excel files
workbook <- createWorkbook(type = "xlsx")
sh1 <- createSheet(workbook,sheetName = "STable6")#sheetName = "NA.all timeSA")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 6: Results of NPMANOVA of North American stegosaurs vs ankylosaurs across time showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh1,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(na.np$res1Omn,sheet = sh1,row.names = FALSE,startRow = 2)

sh2 <- createSheet(workbook,sheetName = "STable7")#sheetName = "NA.LateJ EarlyK SA")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 7: Results of NPMANOVA of North American stegosaurs vs ankylosaurs Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh2,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(na.np$res2Omn,sheet = sh2,row.names = FALSE,startRow = 2)

sh3 <- createSheet(workbook,sheetName = "STable8")#,sheetName = "NA.all timeIS omn")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 8: Results of NPMANOVA of North American iguanodontians and sauropods from all time periods across time showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh3,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(na.np$res3Omn,sheet = sh3,row.names = FALSE,startRow = 2)

sh4 <- createSheet(workbook,sheetName = "STable9")#,sheetName = "NA.LateJ EarlyK IS omn")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 9: Results of NPMANOVA of North American iguanodontians and sauropods from the Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh4,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(na.np$res4Omn,sheet = sh4,row.names = FALSE,startRow = 2)

sh5 <- createSheet(workbook,sheetName = "STable10")#,sheetName = "NA.all timeIS pair")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 10: Results of NPMANOVA of pairwise comparisons between North American iguanodontians and sauropods from the Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh5,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(na.np$res3Pair,sheet = sh5,row.names = FALSE,startRow = 2)

sh6 <- createSheet(workbook,sheetName = "STable11")#,sheetName = "LateJ EarlyK IS pair")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 11: Results of NPMANOVA of pairwise comparisons between North American iguanodontians and sauropods from the Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh6,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(na.np$res4Pair,sheet = sh6,row.names = FALSE,startRow = 2)

# -------------------------------------------------------------------------------------------------------#
# repeat for non-NA
### export degrees of freedom, test-statistics and p-values for each individual test as excel files
sh1 <- createSheet(workbook,sheetName = "STable12")#sheetName = "NA.all timeSA")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 12: Results of NPMANOVA of all stegosaurs vs ankylosaurs across time showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh1,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(all.np$res1Omn,sheet = sh1,row.names = FALSE,startRow = 2)

sh2 <- createSheet(workbook,sheetName = "STable13")#sheetName = "NA.LateJ EarlyK SA")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 13: Results of NPMANOVA of all stegosaurs vs ankylosaurs Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh2,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(all.np$res2Omn,sheet = sh2,row.names = FALSE,startRow = 2)

sh3 <- createSheet(workbook,sheetName = "STable14")#,sheetName = "NA.all timeIS omn")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 14: Results of NPMANOVA of all iguanodontians and sauropods from all time periods across time showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh3,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(all.np$res3Omn,sheet = sh3,row.names = FALSE,startRow = 2)

sh4 <- createSheet(workbook,sheetName = "STable15")#,sheetName = "NA.LateJ EarlyK IS omn")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 15: Results of NPMANOVA of all iguanodontians and sauropods from the Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh4,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(all.np$res4Omn,sheet = sh4,row.names = FALSE,startRow = 2)

sh5 <- createSheet(workbook,sheetName = "STable16")#,sheetName = "NA.all timeIS pair")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 16: Results of NPMANOVA of pairwise comparisons between all iguanodontians and sauropods from the Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh5,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(all.np$res3Pair,sheet = sh5,row.names = FALSE,startRow = 2)

sh6 <- createSheet(workbook,sheetName = "STable17")#,sheetName = "LateJ EarlyK IS pair")
addDataFrame(data.frame("myHeader"=
                          c("Supplementary Table 17: Results of NPMANOVA of pairwise comparisons between North American iguanodontians and sauropods from the Late Jurassic and Early Cretaceous showing degrees of freedom, test-statistic and p-value for each iteration.")),
             sheet = sh6,row.names = FALSE,startRow = 1,col.names = FALSE)
addDataFrame(all.np$res4Pair,sheet = sh6,row.names = FALSE,startRow = 2)

saveWorkbook(workbook,"Supplementary Tables 6-17_NPMANOVA results.xlsx")

# -------------------------------------------------------------------------------------------------------#
# -------------------------------------------------------------------------------------------------------#
### export p-values from various tests that will be used given in the main text
# reported in the results section text:
write.xlsx(na.np$omnRes, "NPMANOVA_res_new.xlsx", sheetName = "NAomnibus",row.names = FALSE)

# reported in tables 2 and 3
write.xlsx(na.np$pairComb3, "NPMANOVA_res_new.xlsx", sheetName = "NApairSvIall", append=T,row.names = T)
write.xlsx(na.np$pairComb4, "NPMANOVA_res_new.xlsx", sheetName = "NApairSvIsel", append=T,row.names = T)

# -------------------------------------------------------------------------------------------------------#
# repeat for all
write.xlsx(all.np$omnRes, "NPMANOVA_res_new.xlsx", sheetName = "allomnibus",append=T,row.names = FALSE)

# reported in tables 4 and 5
write.xlsx(all.np$pairComb3, "NPMANOVA_res_new.xlsx", sheetName = "allpairSvIall", append=T,row.names = T)
write.xlsx(all.np$pairComb4, "NPMANOVA_res_new.xlsx", sheetName = "allpairSvIsel", append=T,row.names = T)

