require(xlsx)
require(readxl)
require(ggplot2)
require(ggpubr)
require(tidyr)
require(plyr)
require(dplyr)
#require(patchwork)
require(colorspace)
require(ggnewscale)
require(e1071)
################################################################################################################################
setwd(dirname(rstudioapi::getActiveDocumentContext()$path)) # sets the working directory automatically to wherever your R script is saved.
mydata <- read_excel("Supplementary Tables 1-5_ecomorph occ Late J-K NA.xlsx", sheet = "STable4", skip=1) # body mass data
timedata <- read_excel("Supplementary Tables 1-5_ecomorph occ Late J-K NA.xlsx", sheet = "STable5", skip=1) # time intervals

masscol<-10 # index of the column containing the log-transformed body mass data
bodgrp<- 4 # index of column containing grouping information to use in the plot
bodDat<-mydata # data frame with body mass data
tme<- timedata # the data frame with the time delineations
youngT<- 7# index for the youngest time boundary for formation
oldT<- 6# index for the oldest time boundary for formation
finetime<-5 # index for the fine time bins
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
# format the data frame(s)
# change column names for given columns to be used in the plotting
colnames(tme)[oldT]<-"old"
colnames(tme)[youngT]<-"young"
colnames(tme)[finetime]<-"finetime"
tme$lab<-ifelse(tme$old == tme$young, 
                paste("(",tme$old, ")", sep=""), 
                paste("(",tme$young, "-", tme$old,")", sep="")   )
tme$lab<-paste( paste(tme$Formation, "Fm", sep=" ")  , tme$Location, tme$lab, sep="\n")
aOrder<-rev(tme$Formation) # get order of formations so that they remain that way
bin_order<-rev(unique(tme$finetime)) # this contains the order (from top to bottom) that plots should be arranged in

fmLabs<-setNames(  tme$lab, tme$Formation   )

colnames(bodDat)[masscol]<-"mass"
colnames(bodDat)[bodgrp]<-"group"
bodDat <- bodDat[!(bodDat$mass == "" | is.na(bodDat$mass)), ] # remove rows with empty data

bodDat<-join(bodDat, data.frame(formation=tme$Formation, mybin=tme$`time bin`, loc= tme$Location, timeBin=tme$finetime), by ='formation', type="left",match="all")
bodDat<-transform(bodDat, formation=factor(formation, levels=aOrder) ) # change order in data frame so factor levels are correct in final plot
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#

#----------------------------------------------------------------------------------------------------------#
# function that creates a faceted plot with body size distributions for each formation from a given time bin
one.bin.body.dist<-function(oneTime, fmLabs, mycol){
  #-----------------------------------------------------------------------------------#
  # plot aesthetics based on clade and overall plot theme, and order of time intervals with most recent first
  mycols<- c("#94C4FF", "#94C4FF", "#49FFFF", "#7BFF76", "#94FF47")
  
  mycols <- lighten( mycols, 0.05, method="absolute" )# the outline/fill for the hulls drawn around clades across time
  names(mycols)<- c( "Middle Jurassic", "Late Jurassic","Jurassic-Cretaceous", "Early Cretaceous", "Late Cretaceous"   )
  
  theme_master<-function(base_size=12, base_family=""){
    theme_classic(base_size=base_size, base_family=base_family)%+replace%
      theme(
        axis.text=element_text(color="black"),
        axis.title.x=element_text(size=16, vjust=-0.5, color="black"),
        axis.title.y=element_text(size=16, angle=90, vjust=3, color="black"),
        axis.text.x=element_text(size=14, color="black"),
        axis.text.y=element_text(size=14, angle=90, vjust=2, color="black"),
        axis.ticks=element_line(color="black"),
        axis.ticks.length=unit(2, "mm"),
        
        axis.line=element_line(color="black", linetype="solid"),
        
        panel.background = element_blank(),
        panel.border=element_rect(fill=NA,color="black", size=1, linetype="solid"),
        
        plot.title=element_text(hjust=0.5, vjust=1.8),
        legend.position = "none",
        panel.grid.minor.y=element_line(color="#DCDCDC", size=0.75),
        panel.grid.major.y=element_line(color="#C0C0C0", size=1),
      )}
  #-----------------------------------------------------------------------------------#
  
  #////////////////////////////////////////////////////////////////////////////////////////////#
  # find skewness of distribution
  skew<-data.frame(formation = oneTime$formation,  skewness=NA)%>% distinct()
  
  for( r in 1:nrow(skew)  ){
    v<-data.frame(formation=oneTime$formation, mass=oneTime$mass) %>%subset(formation==skew$formation[r])
    skew$skewness[r]<-round(skewness(v$mass,1),3) }# this finds the skewness of the distribution for each formation and inputs it into a dataframe
  
  skew$absSkew<-abs(skew$skewness) # absolute value of skewness so that it is clearer when evaluating the type
  
  # determine if distribution is highly (=strong) skewed, moderately skewed, or weakly skewed
  skew<-skew %>% mutate(type = case_when(absSkew >=1 ~ '(strong)',
                                         absSkew >= 0.5 & absSkew < 1 ~ '(moderate)',
                                         TRUE ~ '(weak)'))
  skew$lab<-paste("skew=", skew$skewness, skew$type, sep=" " )
  #////////////////////////////////////////////////////////////////////////////////////////////#
  
  oneTimeplots<-ggplot(data=oneTime, mapping=aes(x=mass, fill=mybin) )+
    geom_histogram( aes(y=..density..), binwidth=0.5, color=c("black"))+
    scale_fill_manual(values=mycols)+
    geom_density(fill=NA, color="black", size=1)+
    scale_y_continuous(expand= expansion(mult = c(0, 0.2)))+
    xlab(bquote(~log[10]~ 'mass ' *'('*g*')'  ))+ # x-axis label
    theme_master()+
    theme(strip.background.x = element_rect( color="black",fill="transparent",size=1 ), strip.text.x = element_text( size = 16 ), 
          strip.text.y = element_blank(), strip.background.y = element_blank()   )+ # strip theme
    geom_label(data = skew, aes(x = Inf, y = Inf, label = paste("s=",skewness, type, sep = " ")), 
               fill=c("#FFFFFF"), size=5, hjust="right",vjust="top") +
    facet_wrap(formation~., drop=T , labeller=as_labeller(fmLabs) , scales="fixed" , ncol=mycol )

  return(oneTimeplots)  }
#----------------------------------------------------------------------------------------------------------#

#----------------------------------------------------------------------------------------------------------#
timePlotList<-as.list(1:length(bin_order))
names(timePlotList)<-bin_order
numcol<-setNames(rev(c(3,3, 3, 4, 3)), bin_order)
for(i in 1:length(bin_order)){
  x<-bin_order[[i]]
  timeSub<- bodDat%>% subset(timeBin==x)
  timeSub$formation<-droplevels(factor(timeSub$formation, levels=aOrder)) # change order in data frame so factor levels are correct in final plot
  timeSub<-timeSub[  with(timeSub, order(formation)), ]
  aPlot<-one.bin.body.dist(oneTime=timeSub, fmLabs=fmLabs, mycol=numcol[[i]])
  timePlotList[[i]]<-aPlot }
#----------------------------------------------------------------------------------------------------------#

ggsave(plot=timePlotList[[1]],"LateK3.svg",device="svg", width=600, height=210-30, units="mm",dpi =1000, scale=0.5)
dev.off()

ggsave(plot=timePlotList[[2]],"LateK2.svg",device="svg", width=800, height=400-30, units="mm",dpi =1000, scale=0.5)
dev.off()

ggsave(plot=timePlotList[[3]],"LateK1.svg",device="svg", width=400, height=210-30, units="mm",dpi =1000, scale=0.5)
dev.off()

ggsave(plot=timePlotList[[4]],"Early K.svg",device="svg", width=400, height=210-30, units="mm",dpi =1000, scale=0.5)
dev.off()

ggsave(plot=timePlotList[[5]],"J-k.svg",device="svg", width=600, height=210-30, units="mm",dpi =1000, scale=0.5)
dev.off()
